"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:libbasic3' }"

OrderedCollection subclass:#ChangeSet
	instanceVariableNames:'changedClasses changeSelectors'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Changes'
!

Object subclass:#ChangeFileReader
	instanceVariableNames:'inputStream parseTree changeAction changeSet selector receiver
		arguments receiverSelector receiverReceiver lineNumber position
		className methodSelector'
	classVariableNames:''
	poolDictionaries:''
	privateIn:ChangeSet
!

Object subclass:#DiffSet
	instanceVariableNames:'changed onlyInReceiver onlyInArg'
	classVariableNames:''
	poolDictionaries:''
	privateIn:ChangeSet
!

ChangeSet::ChangeFileReader subclass:#DolphinPACFileReader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:ChangeSet
!

ChangeSet::ChangeFileReader subclass:#SIFChangeFileReader
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:ChangeSet
!

!ChangeSet class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    used in the changes management to keep track of changes

    [author:]
	Claus Gittinger
"
! !

!ChangeSet class methodsFor:'instance creation'!

forExistingClass:aClass
    "build a changeSet for some given class.
     That does of course not give deltas, but instead reflects the current
     state of the given class.
     It is useful in conjunction with the other utility methods,
     for example, when building patchLists, diffSets etc."

    |changeSet s|

    s := ReadWriteStream on:''.
    aClass fileOutOn:s.
    s reset.
    changeSet := self fromStream:s.
    ^ changeSet

"/    |source changeSet chunk sawExcla lastTimeStamp s change nameSpace|

"/    changeSet := self new.
"/    nameSpace := Smalltalk.
"/
"/    "/ first, a classDefinition change ...
"/    changeSet addClassDefinitionChangeFor:aClass.
"/    "/ are there any class-instVars ?
"/    aClass class instVarNames size > 0 ifTrue:[
"/        changeSet addInstVarDefinitionChangeFor:aClass class.
"/    ].
"/    "/ a comment ?
"/    aClass comment size > 0 ifTrue:[
"/        changeSet addClassCommentChangeFor:aClass.
"/    ].
"/
"/    "/ class methods first ...
"/    aClass class methodDictionary keysAndValuesDo:[:sel :mthd |
"/        changeSet addMethodChange:mthd in:aClass class. 
"/    ].
"/
"/    "/ instance methods ...
"/    aClass methodDictionary keysAndValuesDo:[:sel :mthd |
"/        changeSet addMethodChange:mthd in:aClass.
"/    ].
"/
"/    ^ changeSet

    "
     ChangeSet forExistingClass:ChangeSet
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

forExistingMethods:aCollectionOfMethods
    "build a changeSet for a collection of methods.
     That does of course not give deltas, but instead reflects the current
     state of the given set of methods.
     It is useful in conjunction with the other utility methods,
     for example, when building patchLists, diffSets etc."

    |changeSet stream previousPackage|

    stream := ReadWriteStream on:''.
    aCollectionOfMethods do:[:eachMethod | 
        previousPackage ~= eachMethod package ifTrue:[
            stream nextPutAll:'"{ Package: '''.
            stream nextPutAll:eachMethod package asString.
            stream nextPutAll:''' }"'; cr; cr.
        ].
        eachMethod mclass fileOutMethod:eachMethod on:stream
    ].
    stream reset.
    changeSet := self fromStream:stream.
    ^ changeSet

    "
     ChangeSet forExistingMethods:(Array with:(Array compiledMethodAt:#at:)
                                         with:(Object compiledMethodAt:#at:)
                                         with:(Behavior compiledMethodAt:#compiledMethodAt:) )
    "
!

fromDolphinPACStream:aStream
    "build a changeSet from a stream, containing dolphin pac file chunks."

    ^ self fromDolphinPACStream:aStream while:[:thisChange | true]

!

fromDolphinPACStream:aStream while:aConditionBlock
    "build a changeSet from a stream, containing dolphin pac file chunks."

    |changeSet|

    changeSet := self new.
    self 
        changesFromStream:aStream 
        for:changeSet 
        reader:(DolphinPACFileReader new) 
        do:[:aChange :lineNumberOrNil :posOrNil |
            changeSet add:aChange.
            (aConditionBlock value:aChange) ifFalse:[^ changeSet].
        ].

    ^ changeSet
!

fromFile:aFilename
    |mime stream|

    mime := MIMETypes mimeTypeForFilename:aFilename.
    mime isNil ifTrue:[
        mime := aFilename asFilename mimeTypeOfContents.
        mime isNil ifTrue:[
            self error:'unknown MIME type for file'.
        ].
    ].

    [
        |encoding decoder|

        stream := aFilename asFilename readStream.
        mime = 'text/xml' ifTrue:[
            ^ self fromXMLStream:stream.
        ].
        mime = 'application/x-smalltalk-source-sif' ifTrue:[
            ^ self fromSIFStream:stream.
        ].
        mime = 'application/x-smalltalk-dolphin-package' ifTrue:[
            ^ self fromDolphinPACStream:stream.
        ].            
        encoding := CharacterEncoder guessEncodingOfStream:stream.
        encoding notNil ifTrue:[
            decoder := CharacterEncoder encoderFor:encoding.
            stream := EncodedStream stream:stream encoder:decoder.
            stream skipEncodingChunk.
        ].
        ^ self fromStream:stream.
    ] ensure:[
        stream notNil ifTrue:[stream close].
    ].

    "
     ChangeSet fromFile:('changes')
     ChangeSet fromFile:('patches')
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

fromSIFStream:aStream
    "build a changeSet from a SIF stream, containing chunks
     in smalltalk interchange format.
     Return the changeSet."

    ^ self fromSIFStream:aStream while:[:thisChange | true]
!

fromSIFStream:aStream while:aConditionBlock
    "build a changeSet from a SIF stream, containing chunks
     in smalltalk interchange format.
     Return the changeSet."

    |changeSet|

    changeSet := self new.
    self 
        changesFromStream:aStream 
        for:changeSet 
        reader:(SIFChangeFileReader new) 
        do:[:aChange :lineNumberOrNil :posOrNil |
            changeSet add:aChange.
            (aConditionBlock value:aChange) ifFalse:[^ changeSet].
        ].

    ^ changeSet
!

fromStream:aStream
    "build a changeSet from a stream, containing chunks.
     (i.e. either a classes sourceFile or a change-file).
     Return the changeSet."

    ^ self fromStream:aStream while:[:thisChange | true]

    "
     ChangeSet fromStream:('changes' asFilename readStream)
     ChangeSet fromStream:('patches' asFilename readStream)
     ChangeSet fromStream:(Object source asString readStream)
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

fromStream:aStream while:aConditionBlock
    "build a changeSet from a stream, containing chunks.
     (i.e. either a classes sourceFile or a change-file).
     Return the changeSet."

    |changeSet|

    changeSet := self new.
    self 
        changesFromStream:aStream 
        for:changeSet 
        reader:(ChangeFileReader new)
        do:[:aChange :lineNumberOrNil :posOrNil |
            changeSet add:aChange.
            (aConditionBlock value:aChange) ifFalse:[^ changeSet].
        ].

    ^ changeSet

    "
     ChangeSet fromStream:('changes' asFilename readStream)
     ChangeSet fromStream:('patches' asFilename readStream)
     ChangeSet fromStream:(Object source asString readStream)
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

fromXMLStream:aStream
    "build a changeSet from an XML stream, containing XML definitions.
     Return the changeSet."

    |changeSet builder nameSpace|

    aStream isNil ifTrue:[^ nil].

    changeSet := self new.
    nameSpace := Smalltalk.

    builder := XML::SourceScannerNodeBuilder new.
    builder scanFile:aStream do:[:change | changeSet add: change. ].
    ^ changeSet

    "
     ChangeSet fromXMLStream:('../../goodies/xml/vw/xmlFileInTests/XMLParser.xml' asFilename readStream)
     ChangeSetBrowser 
        openOn:(ChangeSet fromXMLStream:('../../goodies/xml/vw/xmlFileInTests/XMLParser.xml' asFilename readStream))
    "
! !

!ChangeSet class methodsFor:'Compatibility-ST80'!

patches
    ^ #()

    "Created: / 27.10.1997 / 13:52:54 / cg"
! !

!ChangeSet class methodsFor:'Compatibility-VW'!

component: component definition: anObject change: changeSymbol
    "Include indication that a class/namespace was added or removed
     from a CodeComponent." 

    self 
        changed:#'component:definition:change:'
        with:
            ( Array 
                    with: component
                    with: anObject
                    with: changeSymbol
            )

! !

!ChangeSet class methodsFor:'instance creation-private'!

changesFromStream:aStream do:aBlock
    "enumerate changes from a stream and invoke aBlock on each.
     The block is invoked with the change, a lineNumberOrNil and streamPosition arguments. 
     The lineNumber is only valid, if the underlying stream 
     provides line-numbers; otherwise, nil is passed."

    ^ self
        changesFromStream:aStream 
        for:(self new) 
        reader:nil
        do:aBlock

    "
     ChangeSet 
        changesFromStream:('changes' asFilename readStream) 
        do:[:chg | Transcript showCR:chg]
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

changesFromStream:aStream for:aChangeSet reader:aReader do:aBlock
    "enumerate changes from a stream and invoke aBlock on each.
     The block is invoked with the change, a lineNumberOrNil and streamPosition arguments. 
     The lineNumber is only valid, if the underlying stream 
     provides line-numbers; otherwise, nil is passed."

    |chunk lastTimeStamp s change nameSpace package
     lineNumber pos reader|

    nameSpace := Smalltalk.
    package := Class packageQuerySignal query.

    (reader := aReader) isNil ifTrue:[
        reader := ChangeFileReader new.
    ].
    reader changeSet:aChangeSet.
    reader changeAction:aBlock.
    reader inputStream:aStream.

    [aStream atEnd] whileFalse:[
        aStream skipSeparators.
        lineNumber := aStream lineNumber.
        pos := aStream position1Based.
        chunk := aStream nextChunk.
        (chunk notEmptyOrNil) ifTrue:[
            Class nameSpaceQuerySignal answer:nameSpace do:[
                |parser tree ns pkg|

                parser := Parser for:chunk.
                tree := parser 
                            parseExpressionWithSelf:nil 
                            notifying:nil 
                            ignoreErrors:true 
                            ignoreWarnings:true 
                            inNameSpace:nameSpace.

                tree notNil ifTrue:[
                    tree == #Error ifTrue:[
                        change := DoItChange new.
                        change source:chunk.
                        aBlock value:change value:lineNumber value:pos.
                    ] ifFalse:[    
                        "/ if there is any nameSpace directive in there, extract it.
                        ((ns := parser currentNameSpace) notNil 
                        and:[ns ~~ nameSpace]) ifTrue:[
                            nameSpace := ns
                        ].
                        "/ if there is any package directive in there, extract it.
                        ((pkg := parser currentPackage) notNil 
                        and:[pkg ~~ package]) ifTrue:[
                            package := pkg
                        ].
                        Class packageQuerySignal answer:package do:[
                            Class nameSpaceQuerySignal answer:nameSpace do:[
                                "/
                                "/ what type of chunk is this ...
                                "/
                                tree isConstant ifTrue:[
                                    (s := tree evaluate) isString ifTrue:[
                                        (s startsWith:'---- timestamp ') ifTrue:[
                                            lastTimeStamp := s.
                                        ]
                                    ] ifFalse:[
                                        self error:'unexpected change-chunk' mayProceed:true
                                    ]
                                ] ifFalse:[
                                    tree isMessage ifTrue:[
                                        (reader 
                                            changesFromParseTree:tree 
                                            lineNumber:lineNumber
                                            position:pos
                                        ) ifFalse:[
                                            change := DoItChange new.
                                            change source:chunk.
                                            aBlock value:change value:lineNumber value:pos.
                                        ]
                                    ] ifFalse:[
                                        self error:'unexpected change-chunk' mayProceed:true.
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ]
    ].

    "
     ChangeSet fromStream:('changes' asFilename readStream)
     ChangeSet fromStream:('patches' asFilename readStream)
     ChangeSet fromStream:(Object source asString readStream)
    "

    "Created: / 16.2.1998 / 12:19:34 / cg"
    "Modified: / 14.12.1999 / 15:23:16 / cg"
!

skipEncodingChunkOn:stream
    self obsoleteMethodWarning:'use stream>>skipENcodingChunk'.
    stream skipEncodingChunk
! !

!ChangeSet class methodsFor:'queries'!

current
    "ST-80 compatibility: return the current changeSet"

    |p|

    (Project notNil and:[(p := Project current) notNil]) ifTrue:[
        ^ p changeSet
    ].
    ^ #()

    "
     ChangeSet current 
    "
! !

!ChangeSet methodsFor:'Compatibility-ST80'!

changeClass:aClass
    "dummy here"

    "Created: / 4.2.2000 / 18:30:59 / cg"
!

changeSelectors
    "return a collection of all selectors for which changes are in this changeSet"

    |selectors|

    changeSelectors isNil ifTrue:[
        selectors := IdentitySet new.
        self do:[:chg |
            |cls|

            chg notNil ifTrue:[
                chg isMethodChange ifTrue:[
                    selectors add:chg selector
                ]
            ]
        ].
        changeSelectors := selectors.
    ].
    ^ changeSelectors.

    "
     ChangeSet current changeSelectors  
    "
!

changedClasses
    "return a collection of all classes for which changes are in this changeSet"

    |classes|

    changedClasses isNil ifTrue:[
        classes := IdentitySet new.
        self do:[:chg |
            |cls|

            chg notNil ifTrue:[
                cls := chg changeClass.
                cls notNil ifTrue:[
                    cls isNameSpace ifFalse:[
                        classes add:cls
                    ]
                ]
            ]
        ].
        changedClasses := classes.
    ].
    ^ changedClasses.

    "
     ChangeSet current changedClasses  
     ChangeSet current flushChangedClassesCache
    "

    "Modified: / 14-09-2006 / 16:04:18 / cg"
!

component:component definition:anObject change:changeSymbol
    "Include indication that a class/namespace was added or removed
     from a CodeComponent." 

    self 
        changed:#'component:definition:change:'
        with:
            ( Array 
                    with: component
                    with: anObject
                    with: changeSymbol
            )
!

reorganizeSystem
    "dummy here"

    "Created: / 6.2.2000 / 20:45:10 / cg"
! !

!ChangeSet methodsFor:'change & update'!

changed:anAspectSymbol with:aParameter 
    "Allow objects to depend on the ChangeSet class instead of a particular instance 
     of ChangeSet (which may be switched using projects)."

    self == ChangeSet current ifTrue:[
        ChangeSet changed:anAspectSymbol with:aParameter.
    ].
    super changed:anAspectSymbol with:aParameter
! !

!ChangeSet methodsFor:'changes management'!

addClassCommentChangeFor:aClass
    "add a classComment change to the receiver"

    |newChange|

    newChange := ClassCommentChange class:aClass.
    newChange comment:aClass comment.
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:34 / cg"
!

addClassDefinitionChangeFor:aClass
    "add a classDefinition change to the receiver"

    |newChange|

    newChange := ClassDefinitionChange class:aClass source:(aClass definition).
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:37 / cg"
!

addClassRemoveChange:oldClass 
    "add a classRemove change to the receiver"

    |newChange|

    newChange := ClassRemoveChange new className:oldClass name.
    self rememberChangedClass:oldClass.
    oldClass isPrivate ifTrue:[
        self rememberChangedClass:oldClass topOwningClass.
    ].
    ClassRemoveChange::ClassBeingRemovedQuery 
        answer:oldClass
        do:[
            self addChange:newChange
        ]
    "Modified: / 14.11.2001 / 13:35:39 / cg"
!

addClassRenameChangeFrom:oldName to:newName
    "add a classRename change to the receiver"

    |newChange|

    newChange := ClassRenameChange new oldName:oldName newName:newName.
    self rememberChangedClass:(Smalltalk at:newName).
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:41 / cg"
!

addDoIt:aString
    "add a doIt to the receiver"

    |newChange|

    newChange := DoItChange new source:aString.
    self addChange:newChange.

    "Modified: / 14.11.2001 / 13:35:44 / cg"
!

addInstVarDefinitionChangeFor:aClass
    "add an instVarDefinition change to the receiver"

    |newChange|

    newChange := ClassInstVarDefinitionChange 
                        class:aClass
                        source:(aClass name , ' instanceVariableNames:' , aClass instanceVariableString storeString).
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:45 / cg"
!

addMethodCategoryChange:aMethod category:newCategory in:aClass
    "add a methodCategory change to the receiver"

    |newChange|

    newChange := MethodCategoryChange 
                        class:aClass
                        selector:(aClass selectorAtMethod:aMethod)
                        category:newCategory.
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:48 / cg"
!

addMethodChange:aMethod fromOld:oldMethod in:aClass
    "add a method change to the receiver"

    |newChange|

    newChange := MethodChange 
                        class:aClass
                        selector:aMethod selector
                        source:aMethod source
                        category:aMethod category.
    oldMethod notNil ifTrue:[
        newChange previousVersion:oldMethod.
    ].
    self rememberChangedClass:aClass.
    self addChange:newChange.

    "Modified: / 14.11.2001 / 13:35:50 / cg"
!

addMethodChange:aMethod in:aClass
    "add a method change to the receiver"

    |newChange|

    newChange := MethodChange 
                        class:aClass
                        selector:aMethod selector
                        source:aMethod source
                        category:aMethod category.
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:52 / cg"
!

addMethodPrivacyChange:aMethod in:aClass
    "add a methodPrivacy change to the receiver"

    |newChange|

    newChange := MethodPrivacyChange 
                        class:aClass
                        selector:(aClass selectorAtMethod:aMethod)
                        privacy:aMethod privacy.
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 27.8.1995 / 22:55:22 / claus"
    "Modified: / 14.11.2001 / 13:35:55 / cg"
!

addPrimitiveDefinitionsChangeFor:aClass
    "add a primitiveDefinitions change to the receiver"

    |newChange|

    newChange := ClassPrimitiveDefinitionsChange new
                        class:aClass source:(aClass primitiveDefinitionsString).
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:57 / cg"
!

addPrimitiveFunctionsChangeFor:aClass
    "add a primitiveFunctions change to the receiver"

    |newChange|

    newChange := ClassPrimitiveFunctionsChange new
                        class:aClass source:(aClass primitiveFunctionsString).
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:35:59 / cg"
!

addPrimitiveVariablesChangeFor:aClass
    "add a primitiveVariables change to the receiver"

    |newChange|

    newChange := ClassPrimitiveVariablesChange new
                         class:aClass source:(aClass primitiveVariablesString).
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:36:01 / cg"
!

addRemoveSelectorChange:aSelector fromOld:oldMethod in:aClass
    "add a method-remove change to the receiver"

    |newChange|

    newChange := MethodRemoveChange class:aClass selector:aSelector.
    oldMethod notNil ifTrue:[
        newChange previousVersion:oldMethod.
    ].
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 27.8.1995 / 22:55:22 / claus"
    "Created: / 16.2.1998 / 12:47:07 / cg"
    "Modified: / 14.11.2001 / 13:36:04 / cg"
!

addRenameCategoryChangeIn:aClass from:oldCategory to:newCategory
    "add a category rename change to the receiver"

    |newChange|

    newChange := MethodCategoryRenameChange class:aClass.
    newChange oldCategoryName:oldCategory newCategoryName:newCategory.
    self rememberChangedClass:aClass.
    self addChange:newChange

    "Modified: / 14.11.2001 / 13:36:06 / cg"
! !

!ChangeSet methodsFor:'misc'!

addPatch:nameOfPatch
    "ignored for now - allows fileIn of ST-80 patch stuff .."

    ^ self
! !

!ChangeSet methodsFor:'private-accessing'!

addChange:aChange
    aChange timeStamp:(Timestamp now).
    self add:aChange.
    changedClasses notNil ifTrue:[
        changedClasses add:aChange changeClass.
    ].
    changeSelectors notNil ifTrue:[
        aChange isMethodChange ifTrue:[
            changeSelectors add:aChange selector.
        ].
    ].

"/    aChange sendChangeNotificationThroughSmalltalk. 
    self changed:#addChange: with:aChange.

    "Created: / 14.11.2001 / 13:35:11 / cg"
    "Modified: / 14.11.2001 / 13:36:58 / cg"
!

flushChangedClassesCache
    changedClasses := changeSelectors := nil

    "
     ChangeSet current flushChangedClassesCache
    "
!

rememberChangedClass:aClass
    changedClasses notNil ifTrue:[
        changedClasses add:aClass
    ].
!

removeAll:aCollection
    aCollection notEmpty ifTrue:[
        super removeAll:aCollection.
        changedClasses := changeSelectors := nil.
        self changed:#removeAll: with:aCollection.
    ]
!

unrememberChangedClasses
    changedClasses := nil
! !

!ChangeSet methodsFor:'queries'!

includesChangeForClass:aClass
    |nameOfClass|

    nameOfClass := aClass theNonMetaclass name.

    ^ self contains:[:aChange | aChange className = nameOfClass]

    "
     ChangeSet current includesChangeForClass:ChangeSet
     ChangeSet current includesChangeForClass:ChangeSet class
     ChangeSet current includesChangeForClass:SourceCodeManagerUtilities
    "

    "Modified: / 31.10.2001 / 10:58:40 / cg"
!

includesChangeForClass:aClass selector:selector
    |nameOfClass|

    (self changedClasses includes:aClass) ifFalse:[^ false].
    (self changeSelectors includes:selector) ifFalse:[^ false].

    nameOfClass := aClass name.

    ^ self contains:[:aChange | 
                        selector = aChange selector 
                        ifFalse:[
                            false
                        ] ifTrue:[
                            nameOfClass = aChange className
                        ]
                    ]

    "
     ChangeSet current includesChangeForClass:ChangeSet selector:#includesChangeForClass:
    "

    "Created: / 31.10.2001 / 10:26:31 / cg"
    "Modified: / 31.10.2001 / 10:59:49 / cg"
!

includesChangeForClassOrMetaclass:aClass
    |nameOfClass nameOfMetaclass|

    nameOfClass := aClass theNonMetaclass name.
    nameOfMetaclass := aClass theMetaclass name.

    ^ self contains:[:aChange | 
                        |changeClassName|

                        changeClassName := aChange className.
                        changeClassName = nameOfClass or:[changeClassName = nameOfMetaclass]
                    ]
!

includesChangeForClassOrMetaclassOrPrivateClassOf:aClass
    |nameOfClass nameOfMetaclass|

    nameOfClass := aClass theNonMetaclass name.
    nameOfMetaclass := aClass theMetaclass name.

    self do:[:aChange | 
        |changeClassName changeClass|

        changeClassName := aChange className.
        (changeClassName = nameOfClass) ifTrue:[^ true].
        (changeClassName = nameOfMetaclass) ifTrue:[^ true].
        changeClass := aChange changeClass.
        (changeClass notNil
        and:[changeClass isPrivate
        and:[changeClass owningClass == aClass]]) ifTrue:[
            ^ true
        ]
    ].
    ^ false
! !

!ChangeSet methodsFor:'utilities'!

apply
    "apply all changes in the receivers changeSet"

    self do:[:aChange |
	aChange apply
    ]
!

condenseChangesForClass:aClass 
    "remove all changes for aClass
     (i.e. leave changes for other classes)."

    self condenseChangesForClass:aClass package:nil
!

condenseChangesForClass:aClass package:aPackageSymbol
    "remove all changes for aClass and aPackageSymbol
     (i.e. leave methodChanges for other packages).
     This is invoked when a class is checked into the repository."

    self condenseChangesForClass:aClass selector:nil package:aPackageSymbol

    "Modified: / 5.11.2001 / 14:29:22 / cg"
!

condenseChangesForClass:aClass selector:selectorOrNil package:aPackageSymbol
    "remove all changes for aClass/selector and aPackageSymbol
     (i.e. leave methodChanges for other packages).
     If selectorOrNil is nil, all changes for that class are removed;
     otherwise, only changes for the given selector.
     This is invoked when a class is checked into the repository."

    |changesToRemove className metaClassName chgCls|

    changesToRemove := OrderedCollection new.
    className := aClass theNonMetaclass name.
    metaClassName := aClass theMetaclass name.
    self do:[:aChange | 
        |chgClassName chgClass removeThis mClass mthd|

        removeThis := false.
        chgClassName := aChange className.
        (chgClassName = className or:[chgClassName = metaClassName]) ifTrue:[
            removeThis := true
        ] ifFalse:[
            chgCls := aChange changeClass.
            chgCls isNil ifTrue:[
                (chgClassName startsWith:(aClass name,':')) ifTrue:[
                    "a change for a private class of a no-longer present one..."
                        removeThis := true
                ].
            ].

            (chgCls notNil
            and:[chgCls isPrivate
            and:[chgCls topOwningClass == aClass]]) ifTrue:[
                removeThis := true
            ]
        ].
        selectorOrNil notNil ifTrue:[
            (aChange isMethodChange or:[aChange isMethodRemoveChange]) ifFalse:[
                removeThis := false.
            ] ifTrue:[
                aChange selector = selectorOrNil ifFalse:[
                    removeThis := false.
                ].
            ]
        ].

        removeThis ifTrue:[
            aChange isMethodChange ifTrue:[
                mClass := aChange changeClass.
                mClass notNil ifTrue:[
                    mthd := mClass compiledMethodAt:(aChange selector).
                    mthd isNil ifTrue:[
                        "/ mthd does no longer exist
"/ I no longer understand what this was meant for .. (sigh)
"/                        aPackageSymbol notNil ifTrue:[
"/                            removeThis := false
"/                        ]
                    ] ifFalse:[
                        (aPackageSymbol notNil and:[mthd package ~= aPackageSymbol]) ifTrue:[
                            removeThis := false
                        ]
                    ]
                ]
            ].
        ].
        removeThis ifTrue:[
            changesToRemove add:aChange
        ]
    ].
    changesToRemove notEmpty ifTrue:[
        changedClasses := changeSelectors := nil.
        self removeAll:changesToRemove.
        self changed.
        Smalltalk changed:#currentChangeSet with:self.
    ].

    "Modified: / 10-08-2006 / 17:22:13 / cg"
!

condenseChangesForExtensionsInPackage:aPackageSymbol
    "remove all changes for aClass and aPackageSymbol
     (i.e. leave methodChanges for other packages).
     This is invoked when a class is checked into the repository."

    |changesToRemove|

    changesToRemove := OrderedCollection new.

    self do:[:aChange | 
        |removeThis mClass mthd|

        (aChange isMethodChange or:[aChange isMethodRemoveChange]) ifTrue:[
            removeThis := false.
            mClass := aChange changeClass.
            mClass notNil ifTrue:[
                mthd := mClass compiledMethodAt:(aChange selector).
                mthd isNil ifTrue:[
                    aChange isMethodRemoveChange ifTrue:[
                        removeThis := true
                    ].
                ] ifFalse:[
                    mthd package = aPackageSymbol ifTrue:[
                        removeThis := true
                    ]
                ]
            ].
            removeThis ifTrue:[
                changesToRemove add:aChange
            ]
        ].
    ].

    changesToRemove notEmpty ifTrue:[
        changedClasses := changeSelectors := nil.
        self removeAll:changesToRemove.
        self changed.
        Smalltalk changed:#currentChangeSet with:self.
    ].

    "Created: / 05-11-2001 / 14:21:17 / cg"
    "Modified: / 10-08-2006 / 17:22:46 / cg"
!

diffSetsAgainst:anotherChangeSet
    "walk over the receiver and anotherChangeSet,
     add all changes to one of the tree lists:
        onlyInReceiver, onlyInArg or changed,
     each being a changeSet containing corresponding changes.
     WARNING: destructive; could modify both the receiver and the argument"

    |otherChangeIndicesBySelector otherNonMethodChangeIndices changeIndicesBySelector nonMethodChangeIndices
     onlyInReceiver onlyInArg changedMethods
     indexFromChangedMethodsToA indexFromChangedMethodsToB
     "info" ret|

    onlyInReceiver := ChangeSet new.
    onlyInArg      := ChangeSet new.
    changedMethods := ChangeSet new.

    indexFromChangedMethodsToA := OrderedCollection new.
    indexFromChangedMethodsToB := OrderedCollection new.

    otherChangeIndicesBySelector := Dictionary new.
    otherNonMethodChangeIndices := OrderedCollection new.

    "/ these caches reduces square runtime to almost linear...
    anotherChangeSet keysAndValuesDo:[:idxB :aChangeInB |
        |setOfOtherChangeIndicesForThisSelector|

        aChangeInB isMethodChange ifTrue:[
            setOfOtherChangeIndicesForThisSelector := otherChangeIndicesBySelector at:(aChangeInB selector) ifAbsentPut:[OrderedCollection new].
            setOfOtherChangeIndicesForThisSelector add:idxB.
        ] ifFalse:[
            otherNonMethodChangeIndices add:idxB
        ].
    ].

    self keysAndValuesDo:[:idxA :aChangeInA |
        |indicesOfChangesToExplore anyFound ch|

        anyFound := false.

        aChangeInA isMethodChange ifTrue:[
            indicesOfChangesToExplore := otherChangeIndicesBySelector at:(aChangeInA selector) ifAbsent:#()
        ] ifFalse:[
            indicesOfChangesToExplore := otherNonMethodChangeIndices.
        ].

        indicesOfChangesToExplore do:[:idxB |
            |aChangeInB|

            aChangeInB := anotherChangeSet at:idxB.

            (aChangeInA isForSameAs:aChangeInB) ifTrue:[
                anyFound := true.

                "/ also in B - is it different ?
                (aChangeInA sameAs:aChangeInB) ifFalse:[
                    changedMethods add:aChangeInA.
                    indexFromChangedMethodsToA add:idxA.
                    indexFromChangedMethodsToB add:idxB.
                ] ifTrue:[
                    aChangeInA isMethodChange ifTrue:[
                        aChangeInA methodCategory ~= aChangeInB methodCategory ifTrue:[
                            "/ only the category is different;
                            "/ make it a MethodCategory changes.

                            ch := MethodCategoryChange new
                                    className:aChangeInA className
                                    selector:aChangeInA selector
                                    category:aChangeInA methodCategory.
                            self at:idxA put:ch.

                            ch := MethodCategoryChange new
                                    className:aChangeInB className
                                    selector:aChangeInB selector
                                    category:aChangeInB methodCategory.
                            anotherChangeSet at:idxB put:ch.

                            changedMethods add:aChangeInA.
                            indexFromChangedMethodsToA add:idxA.
                            indexFromChangedMethodsToB add:idxB.
                        ]
                    ].
                ]
            ] ifFalse:[
                (aChangeInA sameAs:aChangeInB) ifTrue:[
                    anyFound := true.
                ] ifFalse:[
                ]
            ]
        ].

        anyFound ifFalse:[
            onlyInReceiver add:aChangeInA.
        ]
    ].

    "/ these caches reduces square runtime to almost linear...
    changeIndicesBySelector := Dictionary new.
    nonMethodChangeIndices := OrderedCollection new.

    self keysAndValuesDo:[:idxA :aChangeInA |
        |setOfChangeIndicesForThisSelector|

        aChangeInA isMethodChange ifTrue:[
            setOfChangeIndicesForThisSelector := changeIndicesBySelector at:(aChangeInA selector) ifAbsentPut:[OrderedCollection new].
            setOfChangeIndicesForThisSelector add:idxA.
        ] ifFalse:[
            nonMethodChangeIndices add:idxA
        ].
    ].

    anotherChangeSet keysAndValuesDo:[:idxB :aChangeInB |
        |anyFound indicesOfChangesToExplore|

        anyFound := false.

        aChangeInB isMethodChange ifTrue:[
            indicesOfChangesToExplore := changeIndicesBySelector at:(aChangeInB selector) ifAbsent:#()
        ] ifFalse:[
            indicesOfChangesToExplore := nonMethodChangeIndices.
        ].

        indicesOfChangesToExplore do:[:idxA |
            |aChangeInA idxM|

            aChangeInA := self at:idxA.

            (aChangeInA isForSameAs:aChangeInB) ifTrue:[
                anyFound := true.

                "/ also in B - is it different ?
                (aChangeInA sameAs:aChangeInB) ifFalse:[
                    "/ already there ?
                    idxM := changedMethods findFirst:[:c | c isForSameAs:aChangeInB].
                    idxM == 0 ifTrue:[
                        changedMethods add:aChangeInB.
                        indexFromChangedMethodsToB add:idxB.
                    ] ifFalse:[
                        indexFromChangedMethodsToB at:idxM put:idxB
                    ]
                ]
            ] ifFalse:[
                (aChangeInA sameAs:aChangeInB) ifTrue:[
                    anyFound := true.
                ] ifFalse:[
                ]
            ]
        ].
        anyFound ifFalse:[
            onlyInArg add:aChangeInB.
        ]
    ].

"/    info := OrderedCollection new:(changedMethods size).
"/    changedMethods keysAndValuesDo:[:idx :changedMethod |
"/        info add:(Array 
"/                        with:(indexFromChangedMethodsToA at:idx)
"/                        with:(indexFromChangedMethodsToB at:idx)
"/                 )
"/    ].
    changedMethods := (1 to:changedMethods size) asOrderedCollection collect:[:idx |
                        |cA cB|

                        cA := self at:(indexFromChangedMethodsToA at:idx).
                        cB := anotherChangeSet at:(indexFromChangedMethodsToB at:idx).
                        Array with:cA with:cB
                      ].

    ret := DiffSet new
                changed:changedMethods
                onlyInReceiver:onlyInReceiver
                onlyInArg:onlyInArg.
"/    ret info:info.
    ^ret
! !

!ChangeSet::ChangeFileReader methodsFor:'accessing'!

changeAction:aBlock
    changeAction := aBlock.
!

changeSet:something
    "set the value of the instance variable 'changeSet' (automatically generated)"

    changeSet := something.
!

inputStream:aStream
    inputStream := aStream.
! !

!ChangeSet::ChangeFileReader methodsFor:'helpers'!

addChange:change
    changeAction value:change value:lineNumber value:position.
!

checkReceiverIsGlobalNamed:expectedName
    ^ self variableNameOfReceiver = expectedName
!

classNameOf:aReceiver
    (aReceiver isUnaryMessage 
    and:[aReceiver selector == #class]) ifTrue:[
        ^ (aReceiver receiver name) , ' class'.
    ].
    ^ aReceiver name
!

receiversClassName
    ^ self classNameOf:receiver
!

variableNameOfReceiver

    receiver isVariable ifFalse:[ ^ nil ].
    ^ receiver name
! !

!ChangeSet::ChangeFileReader methodsFor:'reading'!

changesFromParseTree:aTree lineNumber:initialLineNumberOrNil position:initialPositionOrNil
    "given a parse-tree (from parsing some changes source/chunk),
     create changes and evaluate changeAction on each.
     The chnageAction-block is invoked with the change and a lineNumberOrNil as
     arg; the lineNumber is only valid, if the underlying inputStream 
     provides line-numbers; otherwise, nil is passed."

    lineNumber := initialLineNumberOrNil.
    position := initialPositionOrNil.
    parseTree := aTree.

    selector := aTree selector.
    receiver := aTree receiver.
    arguments := aTree arguments.

    receiver isMessage ifTrue:[
        receiverSelector := receiver selector.
        receiverReceiver := receiver receiver.
    ].

    ^ self processChange
! !

!ChangeSet::ChangeFileReader methodsFor:'reading-private'!

handleClassCommentChange
    |change|

    className := self receiversClassName.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].

    change := ClassCommentChange new.
    change className:className comment:(arguments at:1) evaluate.
    change source:(parseTree printString).
    self addChange:change.
    ^ true
!

handleClassDefinitionChange
    |nameSpace change|

    className := (arguments at:1) evaluate.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].
    nameSpace := Class nameSpaceQuerySignal query.
    nameSpace ~~ Smalltalk ifTrue:[
        className := nameSpace name , '::' , className
    ].

    change := ClassDefinitionChange new.
    change className:className; source:(parseTree printString).
    receiver isVariable ifTrue:[
        change superClassName:receiver name.
    ].
    selector keywords with:arguments do:[:kw :arg |
        kw = #'instanceVariableNames:' ifTrue:[
            change instanceVariableNames:arg evaluate.
        ].
        kw = #'classVariableNames:' ifTrue:[
            change classVariableNames:arg evaluate.
        ].
        kw = #'poolDictionaries:' ifTrue:[
            change poolDictionaries:arg evaluate.
        ].
        kw = #'category:' ifTrue:[
            change category:arg evaluate.
        ].
    ].

    change package:(Class packageQuerySignal query).
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            change nameSpaceName:(nameSpace name).
"/        ].
"/
    self addChange:change.
    ^ true
!

handleClassInstanceVariableDefinitionChange
    |change|

    className := self receiversClassName.

"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].
    change := ClassInstVarDefinitionChange new.
    change className:className.
    change source:(parseTree printString).
    self addChange:change.
    ^ true
!

handleMethodCategoryChange
    |change|

    (receiver isMessage
    and:[receiverSelector == #'compiledMethodAt:']) ifFalse:[
        self error:'unexpected change' mayProceed:true.
        ^ false.
    ].
    className := self classNameOf:receiverReceiver.
    className isNil ifTrue:[self halt:'should not happen'].

"/            nameSpace ~~ Smalltalk ifTrue:[
"/                className := nameSpace name , '::' , className
"/            ].
    methodSelector := (receiver arguments at:1) evaluate.
    methodSelector isNil ifTrue:[self halt:'should not happen'].

    change := MethodCategoryChange new.
    change 
        className:className
        selector:methodSelector
        category:(arguments at:1) evaluate.

    self addChange:change.
    ^ true
!

handleMethodCategoryRenameChange
    |change|

    className := self receiversClassName.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].

    change := MethodCategoryRenameChange new.
    change 
        className:className;
        oldCategoryName:(arguments at:1) evaluate
        newCategoryName:(arguments at:2) evaluate.

    self addChange:change.
    ^ true
!

handleMethodChange
    |priv categoryName methodSource changes change parser |

    (selector == #'ignoredMethodsFor:') ifTrue:[
        priv := #ignored.
    ] ifFalse:[
        priv := nil
    ].
    className := self receiversClassName.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].
    ((selector == #'methods') or:[(selector == #'methodsFor')]) ifTrue:[
        categoryName := 'uncategorized'
    ] ifFalse:[
        categoryName := (arguments at:1) evaluate.
    ].
    inputStream skipSeparators.
    lineNumber := inputStream lineNumber.
    position := inputStream position1Based.

    methodSource := inputStream nextChunk.
    changes := OrderedCollection new.

    [methodSource notEmptyOrNil] whileTrue:[
        parser := Parser
                    parseMethodArgAndVarSpecification:methodSource 
                    in:nil 
                    ignoreErrors:true 
                    ignoreWarnings:true
                    parseBody:false.

        parser isNil ifTrue:[
            "/ something wierd ...
            methodSelector := '????'.
        ] ifFalse:[
            methodSelector := parser selector.
        ].

        change := MethodChange new.
        change 
            className:className
            selector:methodSelector
            source:methodSource
            category:categoryName
            privacy:priv.

        self addChange:change.

        inputStream skipSeparators.
        lineNumber := inputStream lineNumber.
        position := inputStream position1Based.
        methodSource := inputStream nextChunk.
    ].
    ^ true
!

handleMethodPrivacyChange
    |change|

    (receiver isMessage
    and:[receiverSelector == #'compiledMethodAt:']) ifFalse:[
        self error:'unexpected change' mayProceed:true.
        ^ false.
    ].

    className := self classNameOf:receiverReceiver.
"/            nameSpace ~~ Smalltalk ifTrue:[
"/                className := nameSpace name , '::' , className
"/            ].
    methodSelector := (receiver arguments at:1) evaluate.

    change := MethodPrivacyChange new.
    change 
        className:className
        selector:methodSelector
        privacy:(arguments at:1) evaluate.

    self addChange:change.
    ^ true
!

handleNameSpaceCreationChange
    |change|

    (self checkReceiverIsGlobalNamed:#Namespace) ifFalse:[
        (self checkReceiverIsGlobalNamed:#NameSpace) ifFalse:[
            self error:'unexpected receiver in nameSpace message' mayProceed:true.
            ^ false
        ].
    ].

    className := (arguments at:1) evaluate.

    change := NameSpaceCreationChange new.
    change name:className.
    self addChange:change.
    ^ true
!

handlePrimitiveChange 
    |change primSource|

    className := self receiversClassName.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].

    inputStream skipSeparators.
    primSource := inputStream nextChunk.

    selector == #'primitiveDefinitions' ifTrue:[
        change := ClassPrimitiveDefinitionsChange new
    ] ifFalse:[
        selector == #'primitiveFunctions' ifTrue:[
            change := ClassPrimitiveFunctionsChange new
        ] ifFalse:[
            change := ClassPrimitiveVariablesChange new
        ]
    ].
    change className:className source:primSource.
    self addChange:change.
    ^ true
!

handleRemoveClassChange
    |change|

    (self checkReceiverIsGlobalNamed:#Smalltalk) ifFalse:[
        self error:'unexpected receiver in remove-class message' mayProceed:true.
        ^ false
    ].

    className := (arguments at:1) name.
"/            nameSpace ~~ Smalltalk ifTrue:[
"/                className := nameSpace name , '::' , className
"/            ].

    change := ClassRemoveChange new.
    change className:className.
    self addChange:change.
    ^ true
!

handleRemoveMethodChange
    |change|

    className := self receiversClassName.
"/        nameSpace ~~ Smalltalk ifTrue:[
"/            className := nameSpace name , '::' , className
"/        ].
    methodSelector := (arguments at:1) evaluate.
    change := MethodRemoveChange new.
    change className:className selector:methodSelector.
    self addChange:change.
    ^ true
!

handleRenameClassChange
    |oldName newName change|

    (self checkReceiverIsGlobalNamed:#Smalltalk) ifFalse:[
        self error:'unexpected receiver in rename-class message' mayProceed:true.
        ^ false.
    ].

    oldName := (arguments at:1) name.
    newName := (arguments at:2) evaluate.

    change := ClassRenameChange new oldName:oldName newName:newName.
    self addChange:change.
    ^ true
!

handleSqueakCommentStamp
    |comment change|

    inputStream skipSeparators.

    comment := inputStream nextChunk.

    className := self receiversClassName.
    change := ClassCommentChange new.
    change className:className comment:comment.
    change source:(parseTree printString).
    self addChange:change.
    ^ true
!

handleVW7ClassDefinitionChange
    |nameSpace change|

    nameSpace := receiver.
    className := (arguments at:1) evaluate.

"/    nameSpace := Class nameSpaceQuerySignal query.
"/    nameSpace ~~ Smalltalk ifTrue:[
"/        className := nameSpace name , '::' , className
"/    ].

    change := ClassDefinitionChange new.
    change className:className; source:(parseTree printString).

    change package:(Class packageQuerySignal query).
    self addChange:change.
    ^ true
!

processChange
    "given a parse-tree (from parsing some changes source/chunk),
     create changes and evaluate changeAction on each.
     The block is invoked with the change and a lineNumberOrNil as
     arg; the lineNumber is only valid, if the underlying stream 
     provides line-numbers; otherwise, nil is passed."

    (selector == #'methods'
    or:[(selector == #'methodsFor')
    or:[(selector == #'methodsFor:')
    or:[(selector == #'methodsForUndefined:')
    or:[(selector == #'publicMethodsFor:')
    or:[(selector == #'privateMethodsFor:')
    or:[(selector == #'methodsFor:stamp:')
    or:[(selector == #'ignoredMethodsFor:')]]]]]]]) ifTrue:[
        ^ self handleMethodChange.
    ].

    selector == #'removeSelector:' ifTrue:[
        ^ self handleRemoveMethodChange.
    ].

    "/ any subclass definiton selector ?
    (Behavior definitionSelectors includes:selector)
    ifTrue:[
        ^ self handleClassDefinitionChange.
    ].

    (selector == #'defineClass:superclass:indexedType:private:instanceVariableNames:classInstanceVariableNames:imports:category:')
    ifTrue:[
        ^ self handleVW7ClassDefinitionChange.
    ].

    selector == #'renameCategory:to:' ifTrue:[
        ^ self handleMethodCategoryRenameChange.
    ].

    (selector == #'category:') ifTrue:[
        ^ self handleMethodCategoryChange.
    ].
    (selector == #'privacy:') ifTrue:[
        ^ self handleMethodPrivacyChange.
    ].

    selector == #'comment:' ifTrue:[
        ^ self handleClassCommentChange.
    ].

    selector == #'commentStamp:prior:' ifTrue:[
        ^ self handleSqueakCommentStamp.
    ].

    selector == #'instanceVariableNames:' ifTrue:[
        ^ self handleClassInstanceVariableDefinitionChange
    ].

    selector == #'removeClass:' ifTrue:[
        ^ self handleRemoveClassChange.
    ].

    selector == #'renameClass:to:' ifTrue:[
        ^ self handleRenameClassChange.
    ].

    selector == #'name:' ifTrue:[
        ^ self handleNameSpaceCreationChange.
    ].

    (selector == #'primitiveDefinitions'
    or:[selector == #'primitiveFunctions'
    or:[selector == #'primitiveVariables']]) ifTrue:[
        ^ self handlePrimitiveChange.
    ].

    ^ false

    "Created: / 16.2.1998 / 13:42:40 / cg"
    "Modified: / 15.12.1999 / 00:29:06 / cg"
! !

!ChangeSet::DiffSet methodsFor:'accessing'!

changed
    "return the value of the instance variable 'changed' (automatically generated)"

    ^ changed
!

changed:something
    "set the value of the instance variable 'changed' (automatically generated)"

    changed := something.
!

changed:changedArg onlyInReceiver:onlyInReceiverArg onlyInArg:onlyInArgArg 
    "set instance variables (automatically generated)"

    changed := changedArg.
    changed isNil ifTrue:[changed := OrderedCollection new].

    onlyInReceiver := onlyInReceiverArg.
    onlyInReceiver isNil ifTrue:[onlyInReceiver := ChangeSet new].

    onlyInArg := onlyInArgArg.
    onlyInArg isNil ifTrue:[onlyInArg := ChangeSet new].
!

onlyInArg
    ^ onlyInArg
!

onlyInArg:something
    onlyInArg := something.
!

onlyInReceiver
    ^ onlyInReceiver
!

onlyInReceiver:something
    onlyInReceiver := something.
! !

!ChangeSet::DiffSet methodsFor:'queries'!

isEmpty
    ^ changed isEmpty
    and:[ onlyInReceiver isEmpty
    and:[ onlyInArg isEmpty ] ]
! !

!ChangeSet::DolphinPACFileReader methodsFor:'reading-private'!

handleCategoriesForClass
    |category change|

    inputStream skipSeparators.
    category := inputStream nextChunk.

    change := ClassCategoryChange new.
    change 
        className:className
        category:category.

    self addChange:change.
    ^ true
!

handleCategoriesForMethod
    |methodSelector categories category change isPrivate isPublic|

    methodSelector := (arguments at:1) evaluate.
    inputStream skipSeparators.
    isPrivate := false.

    categories := OrderedCollection new.
    [
        category := inputStream nextChunk withoutSeparators.
        category notEmpty ifTrue:[
            categories add:category
        ].
        category notEmpty
    ] whileTrue.

    categories size > 1 ifTrue:[
        isPrivate := categories includes:'private'.
        categories remove:'private' ifAbsent:nil.
    ].
    categories size > 1 ifTrue:[
        isPublic := categories includes:'public'.
        categories remove:'public' ifAbsent:nil.
    ].
"/    categories size > 1 ifTrue:[
"/        self halt.
"/    ].

    category := categories first.
    isPrivate ifTrue:[
        category := category , '-private'. 
    ].

    change := MethodCategoryChange new.
    change 
        className:className
        selector:methodSelector
        category:category.

    self addChange:change.
    ^ true
!

processChange
    "given a parse-tree (from parsing some changes source/chunk),
     create changes and evaluate changeAction on each.
     The block is invoked with the change and a lineNumberOrNil as
     arg; the lineNumber is only valid, if the underlying stream 
     provides line-numbers; otherwise, nil is passed."

    (selector == #'categoriesForClass') ifTrue:[
        ^ self handleCategoriesForClass.
    ].
    (selector == #'categoriesFor:') ifTrue:[
        ^ self handleCategoriesForMethod.
    ].
    selector == #'subclass:instanceVariableNames:classVariableNames:poolDictionaries:classInstanceVariableNames:' ifTrue:[
        ^ self handleClassDefinitionChange.
    ].
    ^ super processChange


"/    selector == #named:superclass:indexedInstanceVariables:instanceVariableNames:classVariableNames:sharedPools:classInstanceVariableNames: ifTrue:[
"/        ^ self handleClassDefinitionChange.
"/    ].
"/    (selector == #'key:value:') ifTrue:[
"/        (self checkReceiverIsGlobalNamed:'Annotation') ifTrue:[
"/            ^ self handleAnnotation.
"/        ]
"/    ].
"/    selector == #'method' ifTrue:[
"/        ^ self handleMethodChange:false.
"/    ].
"/    selector == #'classMethod' ifTrue:[
"/        ^ self handleMethodChange:true.
"/    ].
"/
"/    selector == #'initializer' ifTrue:[
"/        ^ true
"/    ].
"/    selector == #'initialize' ifTrue:[
"/        ^ true
"/    ].
"/self halt.
"/
"/    ^ false

    "Created: / 16.2.1998 / 13:42:40 / cg"
    "Modified: / 15.12.1999 / 00:29:06 / cg"
! !

!ChangeSet::SIFChangeFileReader methodsFor:'reading-private'!

handleAnnotation
    |key value lastChange change|

    key := (arguments at:1) evaluate.
    value := (arguments at:2) evaluate.

    key = 'Prerequisite' ifTrue:[
"/        change := SIFPrerequisiteChange new prerequisiteFileName:value.
"/        self addChange:change.
        ^ true
    ].

    lastChange := changeSet last.

    key = 'package' ifTrue:[
        change := ClassCategoryChange new className:(lastChange className) category:value.
        self addChange:change.
        ^ true
    ].
    key = 'category' ifTrue:[
"/        change := MethodCategoryChange new className:(lastChange className) selector:(lastChange selector) category:value.
"/        self addChange:change.
        lastChange category:value.
        ^ true
    ].
    key = 'stamp' ifTrue:[
        "/ lastChange timeStamp:value.
        ^ true
    ].
    ^ false
!

handleClassDefinitionChange
    |nameSpace change|

    (self checkReceiverIsGlobalNamed:#Class) ifFalse:[
        self error:'unexpected receiver in classDefinition message' mayProceed:true.
        ^ false.
    ].
    className := (arguments at:1) evaluate.

    nameSpace := Class nameSpaceQuerySignal query.
    nameSpace ~~ Smalltalk ifTrue:[
        className := nameSpace name , '::' , className
    ].

    change := ClassDefinitionChange new.
    change className:className; source:(parseTree printString).

    change package:(Class packageQuerySignal query).
    self addChange:change.
    ^ true
!

handleMethodChange:isMeta
    |categoryName methodSource change parser |

    className := self receiversClassName.
    isMeta ifTrue:[
        className := className , ' class'
    ].

    categoryName := 'uncategorized'.

    inputStream skipSeparators.
    lineNumber := inputStream lineNumber.
    position := inputStream position1Based.

    methodSource := inputStream nextChunk.

    parser := Parser
                parseMethodArgAndVarSpecification:methodSource 
                in:nil 
                ignoreErrors:true 
                ignoreWarnings:true
                parseBody:false.

    parser isNil ifTrue:[
        "/ something wierd ...
        methodSelector := '????'.
    ] ifFalse:[
        methodSelector := parser selector.
    ].

    change := MethodChange new.
    change 
        className:className
        selector:methodSelector
        source:methodSource
        category:categoryName
        privacy:nil.

    self addChange:change.
    ^ true
!

processChange
    "given a parse-tree (from parsing some changes source/chunk),
     create changes and evaluate changeAction on each.
     The block is invoked with the change and a lineNumberOrNil as
     arg; the lineNumber is only valid, if the underlying stream 
     provides line-numbers; otherwise, nil is passed."

    (selector == #'interchangeVersion:') ifTrue:[
        ^ true
    ].
    selector == #named:superclass:indexedInstanceVariables:instanceVariableNames:classVariableNames:sharedPools:classInstanceVariableNames: ifTrue:[
        ^ self handleClassDefinitionChange.
    ].
    (selector == #'key:value:') ifTrue:[
        (self checkReceiverIsGlobalNamed:'Annotation') ifTrue:[
            ^ self handleAnnotation.
        ]
    ].
    selector == #'method' ifTrue:[
        ^ self handleMethodChange:false.
    ].
    selector == #'classMethod' ifTrue:[
        ^ self handleMethodChange:true.
    ].

    selector == #'initializer' ifTrue:[
        ^ true
    ].
    selector == #'initialize' ifTrue:[
        ^ true
    ].

    ^ false

    "Created: / 16.2.1998 / 13:42:40 / cg"
    "Modified: / 15.12.1999 / 00:29:06 / cg"
! !

!ChangeSet class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/ChangeSet.st,v 1.133 2006-09-14 14:04:26 cg Exp $'
! !
